// Minimal critical imports - everything else is lazy loaded
// Import polyfills first
import './polyfills';

import React, { StrictMode } from "react";
import { createRoot } from "react-dom/client";
// Import App directly to remove loading screen
import App from "./App";

// Import CSS with low priority - patterns already applied via critical inline CSS
import "./index.css";

// Lazy load optimization utilities after main content renders
const registerServiceWorkerModule = import('./lib/register-sw').then(module => ({
  registerServiceWorker: module.default,
  prefetchResources: module.prefetchResources,
  sendMessageToServiceWorker: module.sendMessageToServiceWorker,
  optimizeImage: module.optimizeImage
}));

// Optimization utilities loaded on demand after critical content renders
const mobileOptModule = import('./lib/optimize-for-mobile').then(module => ({
  initMobileOptimizations: module.initMobileOptimizations,
  isMobileDevice: module.isMobileDevice,
  isSlowConnection: module.isSlowConnection,
  isLowMemoryDevice: module.isLowMemoryDevice,
  getBatteryInfo: module.getBatteryInfo,
  getConnectionInfo: module.getConnectionInfo
}));

// Error boundary for the entire app
function logError(error: Error) {
  console.error('Application Error:', error);
}

// Lightweight performance metrics before any heavy operations
function measurePerformanceMetrics() {
  // Function to measure First Contentful Paint
  function measureFCP() {
    if (!window.performance || !window.PerformanceObserver) return;
    
    const observer = new PerformanceObserver((entryList) => {
      for (const entry of entryList.getEntries()) {
        console.log(`FCP: ${entry.startTime}ms`);
        observer.disconnect();
      }
    });
    
    observer.observe({ type: 'paint', buffered: true });
  }
  
  // Function to measure Largest Contentful Paint
  function measureLCP() {
    if (!window.performance || !window.PerformanceObserver) return;
    
    const observer = new PerformanceObserver((entryList) => {
      for (const entry of entryList.getEntries()) {
        console.log(`LCP: ${entry.startTime}ms`);
      }
    });
    
    observer.observe({ type: 'largest-contentful-paint', buffered: true });
  }
  
  // Measure key performance metrics
  measureFCP();
  measureLCP();
  
  // Add performance markers for metrics tracking
  if (window.performance) {
    // Mark the start of the app initialization
    performance.mark('app-init-start');
  }
}

// Critical - run performance metrics immediately
measurePerformanceMetrics();

// Handle lazy resource loading and critical path optimization
// Note that functions are not invoked immediately to reduce initial load time
const setupResourceOptimizations = async () => {
  // Load modules asynchronously
  const [swModule, mobileModule] = await Promise.all([
    registerServiceWorkerModule,
    mobileOptModule
  ]);
  
  const { 
    registerServiceWorker, 
    prefetchResources 
  } = swModule;
  
  const {
    initMobileOptimizations,
    isMobileDevice,
    isSlowConnection,
    isLowMemoryDevice
  } = mobileModule;
  
  // Enhanced mobile performance optimizations - now with lazy-loaded dependencies
  const optimizeForMobile = () => {
    // Initialize all mobile optimizations with the dedicated utilities
    initMobileOptimizations();
    
    // Basic optimizations that don't depend on expensive checks
    const lazyLoadResources = () => {
      const lazyImages = document.querySelectorAll('[data-src]');
      
      if ('IntersectionObserver' in window) {
        const imageObserver = new IntersectionObserver((entries) => {
          entries.forEach(entry => {
            if (entry.isIntersecting) {
              const lazyImage = entry.target as HTMLImageElement;
              if (lazyImage.dataset.src) {
                lazyImage.src = lazyImage.dataset.src;
                lazyImage.removeAttribute('data-src');
                imageObserver.unobserve(lazyImage);
              }
            }
          });
        });
        
        lazyImages.forEach(image => {
          imageObserver.observe(image);
        });
      } else {
        // Fallback for browsers without IntersectionObserver
        setTimeout(() => {
          lazyImages.forEach(image => {
            if ((image as HTMLImageElement).dataset.src) {
              (image as HTMLImageElement).src = (image as HTMLImageElement).dataset.src || '';
            }
          });
        }, 1000);
      }
    };
    
    // Deferred prefetching after idle
    const deferredPrefetch = () => {
      if ('requestIdleCallback' in window) {
        (window as any).requestIdleCallback(() => {
          prefetchResources([
            '/favicon.png',
            '/Small logo.png',
          ]);
        }, { timeout: 5000 });
      } else {
        setTimeout(() => {
          prefetchResources([
            '/favicon.png',
            '/Small logo.png',
          ]);
        }, 2000);
      }
    };
    
    // Init lazy loading on page load
    window.addEventListener('load', lazyLoadResources);
    
    // Run prefetch after a delay
    setTimeout(deferredPrefetch, 1500);
    
    // Only run device-specific optimizations after a delay
    setTimeout(() => {
      if (isMobileDevice()) {
        // Enable gesture navigation for mobile users
        if ('ontouchstart' in document.documentElement) {
          document.documentElement.classList.add('touch-enabled');
        }
        
        // For low memory devices, optimize resource usage
        if (isLowMemoryDevice()) {
          // Use minimal animations and effects on low memory devices
          document.documentElement.classList.add('low-memory');
        }
      }
    }, 1000);
  };
  
  // Register Service Worker with minimal initial options
  registerServiceWorker({
    immediate: false, // Delay to prioritize UI rendering
    swPath: '/sw.js',
    onSuccess: () => {
      console.log('🚀 Service worker registered');
    }
  });
  
  // Apply mobile optimizations after a delay
  setTimeout(() => {
    // Access imported modules via local variables to avoid undefined errors
    if (typeof initMobileOptimizations === 'function') {
      optimizeForMobile();
    }
  }, 500);
};

// Mount React app with loading indicator and error handling
try {
  const rootElement = document.getElementById("root");

  if (!rootElement) {
    throw new Error("Root element (#root) not found in the document. Cannot mount React application.");
  }
  
  const root = createRoot(rootElement);

  // Render the app directly without a loading screen
  root.render(
    <StrictMode>
      <App />
    </StrictMode>
  );

  // After render, initialize non-critical optimizations
  setTimeout(() => {
    setupResourceOptimizations().catch(err => {
      console.error('Failed to initialize optimizations:', err);
    });
  }, 300);

  // Mark app rendering complete for performance measurement (after delay)
  setTimeout(() => {
    if (window.performance) {
      performance.mark('app-init-end');
      performance.measure('app-initialization', 'app-init-start', 'app-init-end');
      
      const appInitMeasure = performance.getEntriesByName('app-initialization')[0];
      console.log(`⏱️ App initialization time: ${appInitMeasure.duration.toFixed(2)}ms`);
    }
  }, 1000);

  console.log("✅ React application mounting started");
  
} catch (error) {
  logError(error as Error);
  // Display error message in the DOM if possible
  const errorElement = document.createElement('div');
  errorElement.style.padding = '20px';
  errorElement.style.color = 'red';
  errorElement.textContent = 'Failed to initialize application. Please check the console for details.';
  document.body.appendChild(errorElement);
}